/*--------------------------------------------------------------*/
/*            CONTROL DE TRACCIÓN VEHÍCULO 2X2                  */
/*--------------------------------------------------------------*/
/*                                                              */
/* FECHA: JUNIO DE 2017                                         */
/* Grado en ingeniería eléctronica industrial y automática      */
/* Universidad de Valladolid                                    */
/*                                                              */
/*AUTOR: Roberto Jiménez Peralta                                */
/*       Iván Minguela Benito                                   */
/*CONTACTO:                                                     */
/*       roberto_velayos@hotmail.com                            */
/*--------------------------------------------------------------*/

#include "Control_de_traccion.h"

//---CONFIGURACIÓN DE LOS PINES, DE LAS INTERRUPCIONES Y DEL PUERTO SERIE---///
void setup()
{
  //  Se configura el puerto serie
  Serial.begin(9600);
  //  Se configuran los pines como entradas, salidas y se activan las resistencia interna pull-up, LOS PINES ANALÓGICOS NO HACE FALTA CONFIGURARLOS COMO ENTRADAS O COMO SALIDAS
  pinMode(HALL_TRASERO, INPUT); // Declaramos el pin donde se conecta el sensor hall trasero como entrada. Es analógico no es necesario configurarlo como entrada o salida
  pinMode(HALL_DELANTERO, INPUT); // Declaramos el pin donde se conecta el sensor hall delantero como entrada.
  pinMode(BOTON_ON_OFF, INPUT_PULLUP); // Declaramos el pin del botón como entrada y habilitamos la resistencia pull-up interna
  pinMode(BOTON_2X2_2X1, INPUT_PULLUP); // Declaramos el pin del botón como entrada y habilitamos la resistencia pull-up interna

  pinMode(LED_2X2_2X1, OUTPUT); //declaramos el pin del led como salida
  pinMode(LED_ON_OFF, OUTPUT); //declaramos el pin del led como salida
  pinMode(TRACCION, OUTPUT); //declaramos el pin del led como salida
  pinMode(controlador_delantero, OUTPUT); // Declaramos el pin al que se conectará el motor delantero como salida
  pinMode(tension_vumetro, OUTPUT);

  /*CONFIGURACIÓN DE LAS INTERRUPCIONES*/
  cli(); //deshabilitamos las interrupciones globales
  EIMSK |= (1 << INT0); //Declaramos el uso de las interrupcion 0
  EICRA |= ((1 << ISC01) | (1 << ISC00)); //Definimos que la deteccion sea por flanco ascendente. Tambien podemos usar cualquiera de los dos o "EICRA" o "MCUCR" -> MCUCR |= ((1<<ISC01) |(1<<ISC00));
  PCICR |= (1 << PCIE1); // SE HABILITA INTERRUPCION POR CAMBIO DE PIN Y AHORA SE UBICARA LA INTERRUPCION 1 ENTRE LOS PINES PCINT14-8 CON EL CAMBIO DE PIN AHORA LA DETECCIÓN SERÁ POR CAMBIO DE ESTADO YA SEA ASCENDENTE O DESCENDENTE Y ESTO NO SE PUEDE CAMBIAR
  PCMSK1 |= (1 << PCINT8); //SELECCIONAMOS EN PIN PCINT 11 PARA QUE SEA LA INTERRUPCIÓN 1 QUE QUIVALE AL PIN A3.
  sei(); // habilitamos las interrupciones globales

  lcd.begin (16, 2); //se inicializa la pantalla LCD de 16 caracteres y dos filas
  lcd.setBacklightPin(BACKLIGHT_PIN, POSITIVE); //Se enciende la luz de fondo de la pantalla
  lcd.setBacklight(LED_ON);

  //Se crean los nuevos caracteres
  lcd.createChar (0, bat_1);
  lcd.createChar (1, bat_2);

  lcd.setCursor(4, 0);
  lcd.print("WELCOME");
  lcd.setCursor(5, 1);
  lcd.print("VELO");
  delay (2000);

  //La lectura de voltaje tiene mucho ruido, por lo que se va a usar la media móvil para filtrar el ruido
  for (int i = 0; i < SAMPLES_TO_AVERAGE; i++)
  {
    volatile int voltaje = analogRead (lectura_voltaje);

    voltaje_leido = map (voltaje, 0, 1023, 0, 500); //Se mapea entre 0-500 para despues dividir entre 100 y obtener un valor con dos difras decimales
    voltaje_leido = voltaje_leido / 100;            //Se deja el valor con dos decimales
    voltaje_filtrado = voltaje_filtrado + (voltaje_leido - voltaje_filtrado)/ SAMPLES_TO_AVERAGE;  //Se emplea la fórmula de la media móvil acumulativa
  }
  lcd.clear();
}

/*FUNCIONES A LAS QUE SE LLAMA CON LAS INTERRUPCIONES*/
ISR(PCINT1_vect) // SENSOR TRASERO AL PIN 3
// Rutina de servicio de interrupción. Cada PCInt 8..14
//UN CAMBIO EN LOS PINES ANALÓGICOS DEL 0 AL 5 generará una interrupción: pero esto
//será siempre la misma rutina de interrupción
{
  // handle pin change interrupt for A0 to A5 here
  if (digitalRead(HALL_TRASERO) == HIGH)// MIRAR SI SE PUEDE PONER COMO IF (PINC & (1<<PINC3) == 1) QUE SUPUESTAMENTE REALIZARÍA LO MISMO ES SI EL PIN A3 HA CAMBIADO Y ESTÁ A 1
  {
    /* PCINT11 HA TENIDO UN FLANCO POSITIVO */
    CONTADOR_TRASERO++;
    rpm++; //Se cuentan las revoluciones por minuto para calcular la velocidad
  }
}

ISR(INT0_vect) // SENSOR DELANTERO AL PIN 2
{
  CONTADOR_DELANTERO++; // AUMENTAMOS EL CONTADOR DE FLANCOS ASCENDENTES DE LA RUEDA DELANTERA
}
/*FIN DE FUNCIONES A LAS QUE SE LLAMA CON LAS INTERRUPCIONES*/
/*---------------------------------------------*/

//---PROGRAMA PRINCIPAL---//
void loop()
{
  SELECCION_DEL_MODO();  //Se selecciona el modo de funcionamiento en funcion de los interruptores

  CALCULO_DIFERENCIA();  //Se calcula la diferencia de pulsos de la rueda delantera y trasera

  muestra_velocidad();   //Se muestra a la velocidad a la que se esta circulando por la pantalla LCD

  mostrar_potencia ();   //Se muestra la potencia que se está enviando a la rueda delantera

  leer_tension ();      //Funcion que lee el voltaje restante en los ultracondensadores

  //muestra_tension();   //Se muestra el porcentaje de tension en los ultracondensadores por si se usara una pantalla de 20x4
  //que se podria mostrar mas parametros, aunque ahora no fuera necesario por poder usar el vumetro
}
//---FIN DEL PROGRAMA PRINCIPAL---//

/* CALCULO DE LA DIFERENCIA DE PULSOS */ // SIN TESTEAR
/*
  void CALCULO_DIFERENCIA_DE_PULSOS()
  {
  if (tiempo_diferente >= 500000)
  {
    if (velocidad_rueda_delantera_final <= 2)
    {
      diferencia_pulsos_limite = 1;
    }
    else if (2 < velocidad_rueda_delantera_final <= 4.5)
    {
      diferencia_pulsos_limite = 2;
    }
    else if (4.5 < velocidad_rueda_delantera_final <= 9)
    {
      diferencia_pulsos_limite = 3;
    }
    else if (9 < velocidad_rueda_delantera_final <= 12)
    {
      diferencia_pulsos_limite = 4;
    }
  }

  else if (342857.14 <= tiempo_diferente < 500000)
  {
    diferencia_pulsos_limite = 3;
  }
  else if (230769.24 <= tiempo_diferente < 342857.14)
  {
    diferencia_pulsos_limite = 3;
  }
  else if (171428.57 <= tiempo_diferente < 230769.24)
  {
    diferencia_pulsos_limite = 2;
  }
  else if (137931.03 <= tiempo_diferente < 171428.57)
  {
    diferencia_pulsos_limite = 2;
  }
  }
*/
/* FIN DEL CALCULO DE LA DIFERENCIA DE PULSOS */



