//--- LIBRERÍAS UTILIZADAS --- //

#include <avr/interrupt.h> // Librería necesaria para el uso de las interrupciones
#include <avr/io.h> // Librería necesaria para la definición de entradas y salidas y para poder realizar el cambio de pin de la interrupción

#include <Wire.h> //Librería necesaria para la comunicacion por protocolo I2C con el modulo soldado a la pantalla
#include <LiquidCrystal_I2C.h> //Libreria para utilizar la pantalla LCD con el protocolo I2C
// --------------------------- //

//---ASIGNACIÓN DE LOS PINES UTILIZADOS---//
const int HALL_DELANTERO = 2; //Sensor de efecto hall delantero, INTERRUPCIÓN 0
const int HALL_TRASERO = A0; //Sensor de efecto hall trasero EN EL PIN A0, INTERRUPCIÓN 1
const int controlador_delantero = 5; //Pin al que se conectará el controlador delantero
const int tension_vumetro = 9; //Pin al que se ha conectado la entrada del vumetro

//se van a añadir solo dos botones y borrar los demas
const int BOTON_ON_OFF = 8;
const int BOTON_2X2_2X1 = 7;
//cuando este activado el on, el funcionamiento depende del otro es decir del 2x2 o el 2x1
//lo mismo se va a hacer con los LED
const int LED_ON_OFF = 12;
const int LED_2X2_2X1 = 11;
const int TRACCION = 13;

const int Regulador_de_potencia = 3; //Pin del regulador es el A3(Analog IN), COMENTADO
const int Acelerador = 2; // Pin del acelerador es el A2(Analog in)
const int lectura_voltaje = 1; //Pin por donde se lee la carga de los ultracondensadores
int voltaje_modificado; //Variable donde se guarda el porcentaje de carga que queda
int bateria_baja = 0; //Variable que indica si el convertidor no puede aumentar la tension hasta 36 V 
                      //Cero cuando esta bien, 1 cuando esta baja y 2 cuando ya no pueden funcionar los motores
int volt_traducido_vumetro; //Voltaje a sacar por el pin 9 de arduino para que cuando haya 18 V, en la salida haya 0 V
                      
float voltaje_leido = 0; //Voltaje entre 0 y 5 voltios que se tiene en el pin de lectura
float voltaje_filtrado = 0;
#define SAMPLES_TO_AVERAGE 15
//----------------------------------------//

//---VARIABLES UTILIZADAS---//
int FLAG_0 = 0; // Bandera para solo realizar una vez ciertas acciones cuando el MODO_0 está activado o desactivado
int FLAG_1 = 0; // Bandera para solo realizar una vez ciertas acciones cuando el MODO_1 está activado o desactivado
int FLAG_2 = 0; // Bandera para solo realizar una vez ciertas acciones cuando el MODO_2 está activado o desactivado
int FLAG_3 = 0; // Bandera para solo realizar una vez ciertas acciones cuando el MODO_3 está activado o desactivado

int MODO_0 = 0; //CUANDO MODO_0 = 1, NOS ENCONTRAMOS EN EL MODO_0 --> 2X1 OFF
int MODO_1 = 0; //CUANDO MODO_1 = 1, NOS ENCONTRAMOS EN EL MODO_1 --> 2X1 ON
int MODO_2 = 0; //CUANDO MODO_2 = 1, NOS ENCONTRAMOS EN EL MODO_2 --> 2X2 OFF
int MODO_3 = 0; //CUANDO MODO_3 = 1, NOS ENCONTRAMOS EN EL MODO_3 --> 2X2 ON

int PRIMERA_VEZ_MODO_0 = 0; // Variable que indica que el MODO_0 a pasado de inactivo a activo
int UNA_VEZ = 0; // Variable para mantener durante 3 segundos la señal de Tracción Total
int BLOQUEO = 0; // Variable con la que en Funcionamiento Tracción Total, se indica que si en 3 segundos se produce funcionamiento normal se ignora.

int valor_regulador = 0; // Variable en la que se almacena el valor del regulador de potencia
double valor_nuevo = 1.0; // Variable con la que indicaremos el valor de nuestro regulador ente 0 y 1 con escalones de 0.1
int valor_acelerador = 0; // Variable en la que se almacena el valor del acelerador

int valor_acelerador_traducido = 0; // Variable en la que se almacena el valor de tensión que se deberá de mandar al controlador delantero
double valor_acelerador_traducido_float = 0.0; // Variable en la que se almacena el valor de tensión que se deberá de mandar al controlador delantero pero del tipo coma flotante
double proporcion_acelerador = 0.8; //Este valor sería ajustable por el piloto vía software. Es el valor por el cual se multiplica la señal en modo Tracción Proporcional
int diferencia = 0; // variable en la que guardamos la diferencia entre los contadores de flancos ascendentes entre la rueda trasera y la rueda delantera
double diferencia_filtrada = 0.0; // Variable en la que guardamos la diferencia entre los contadores de los flancos ascendentes de la rueda trasera y los de la rueda delantera, depués del filtro

const long tiempo_calculo_diferencia = 200000; // Tiempo en el que se cuenta los flancos positivos para después realizar la comparación de ellos. LA COMPARACIÓN DE FLANCOS SE REALIZA CADA 0.5 SEGUNDOS
long tiempo_diferente = 0; // Variable en la que guardamos el tiempo en que se tarda en entrar de nuevo en el bucle de comparación de velocidades desde la última vez
long tiempo_anterior = 0; // Tiempo en el que se realizó el último bucle
unsigned long tiempo_actual = 0; // Tiempo actual del programa

volatile int CONTADOR_TRASERO; // Contador de la rueda trasera en el que se contabiliza el número de flancos ascendentes detectados en la señal que manda el sensor hall trasero.
volatile int CONTADOR_DELANTERO; // Contador de la rueda delantera en el que se contabiliza el número de flancos ascendentes detectados en la señal que manda el sensor hall delantero.
int PULSOS_LIMITE = 12; // PULSOS LÍMITE CON LOS QUE PARTIMOS INICIALMENTE HASTA LOS CUALES CONTARAN LOS CONTADORES SI NO SE SUPERA EL MEDIO SEGUNDO, ESTE SEMANTIENE CONSTANTE
int diferencia_pulsos_limite = 3; // DIFERENCIA DE FLANCOS ASCENDENTES CON LA CUAL SE TOMARÁN LAS DECISIONES.

volatile int velocidad_rueda_trasera = 0; // Aquí se guarda el contaje de flancos ascendentes de la rueda trasera
volatile int velocidad_rueda_delantera = 0; // Aquí se guarda el contaje de flancos ascendentes de la rueda delantera
double velocidad_rueda_trasera_final = 0.0; // Contaje de flancos ascendentes de la rueda trasera filtrado
double velocidad_rueda_delantera_final = 0.0; // Contaje de flancos ascendentes de la rueda delantera filtrado
double velocidad_rueda_trasera_final_anterior = 0.0; // Cantidad de flancos ascendentes de la rueda trasera filtrada contabilizados en la anterior comparación de velocidades
double velocidad_rueda_delantera_final_anterior = 0.0; // Cantidad de flancos ascendentes de la rueda delantera filtrada contabilizados en la anterior comparación de velocidades

long t_actual = millis (); //se guarda en esta variable el tiempo actual para saber la diferencia entre los pulsos
volatile int rpm = 0; //variable donde se va a guardar la velocidad en rpm de la rueda delantera
float vel_lineal = 0; //Se guarda la velocidad lineal en esta variable

float pi = 3.1416; // Número PI
float DIAMETRO_RUEDA = 0.4064; // Diámetro de la rueda
float velocidad_RD_float = 0.0; // SEÑAL A COMANDAR AL MOTOR DELANTERO EN COMA FLOTANTE
int velocidad_RD_int = 0; // SEÑAL A COMANDAR AL MOTOR DELANTERO (ENTERO)


unsigned long tiempo_anterior_PID = 0.0; // Tiempo en el que finalizó el último control realizado por el PID
unsigned long Ahora_PID = 0.0; // tiempo en el que se está volviendo a ejecutar el nuevo control del PID
volatile double Error = 0.0; // Diferencia de flancos ascendentes filtrados entre la rueda trasera y delantera
volatile double salida = 0.0, errorDer = 0.0, Error_anterior = 0.0, errorsum = 0.0, errorProp = 0.0; // Definición de distintas variables para la realización el PID
double kp = 0.08, ki = 0.9, kd = 10000; // Término proporcional, integral y derivativo
double dT = 0.0; // Tiempo que tarda en volverse a ejecutar el control
double dT_distinto = 0.0; // Tiempo que tarda en volverse a ejecutar la comparación de flancos ascendentes para los filtros de las señales de los sensores Hall
unsigned long tiempo_anterior_mantener_traccion = 0.0; // tiempo anterior para mantener la señal al motor delantero en caso de pérdida de tracción de la rueda trasera
unsigned long tiempo_mantener_traccion = 0.0; // tiempo para mantener la señal al motor delantero en caso de pérdida de tracción de la rueda trasera

//---VARIABLES PARA IDENTIFICAR CUANDO SE DA TENSÍON AL MOTOR DELANTERO UTILIZANDO EL PID Y CUANDO NO---//

boolean Automatico = false; // INICIALIZAMOS LA VARIABLE
boolean nuevo_Automatico = false; // INICIALIZAMOS LA VARIABLE
#define FuncionamientoManual 0 // SE PROPORCIONA TENSIÓN MÍNIMA O PROPORCIONAL AL ACELERADOR AL MOTOR DELANTERO
#define FuncionamientoAutomatico 1 // SE PROPORCIONA TENSÍON A TRAVÉS DEL PID AL MOTOR DELANTERO

//--- ---//
//----------------------------------------//
//---DECLARACION DE LA PANTALLA LCD Y SUS VARIABLES---//
#define I2C_ADDR    0x3F  //Direccion comprobada con otro sketch para saber la direccion de memoria del modulo 0x3F 0x27
#define BACKLIGHT_PIN  3
#define En_pin  2
#define Rw_pin  1
#define Rs_pin  0
#define D4_pin  4
#define D5_pin  5
#define D6_pin  6
#define D7_pin  7

#define  LED_OFF  LOW
#define  LED_ON  HIGH

LiquidCrystal_I2C  lcd(I2C_ADDR, En_pin, Rw_pin, Rs_pin, D4_pin, D5_pin, D6_pin, D7_pin);

//--CREACIÓN DEL CARACTER BATERIA--//
byte bat_1[8] = {                     //Primer caracter de bateria  
B00100,
B11111,
B10000,
B10100,
B11110,
B10100,
B10000,
B11111,
};
byte bat_2[8] = {                     //Segundo caracter de bateria
B00100,
B11111,
B00001,
B00001,
B01101,
B00001,
B00001,
B11111,
};

